package xyz.lychee.lagfixer.libs.adventure.platform.bukkit;

import com.viaversion.viaversion.api.connection.UserConnection;
import java.util.Collection;
import java.util.Locale;
import java.util.function.Function;
import java.util.function.Supplier;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.Plugin;
import org.bukkit.util.Vector;
import xyz.lychee.lagfixer.libs.adventure.bossbar.BossBar;
import xyz.lychee.lagfixer.libs.adventure.identity.Identity;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.Facet;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.FacetAudience;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.FacetAudienceProvider;
import xyz.lychee.lagfixer.libs.adventure.platform.viaversion.ViaFacet;
import xyz.lychee.lagfixer.libs.adventure.pointer.Pointers;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class BukkitAudience extends FacetAudience<CommandSender> {
   static final ThreadLocal<Plugin> PLUGIN = new ThreadLocal();
   private static final Function<Player, UserConnection> VIA = new BukkitFacet.ViaHook();
   private static final Collection<Facet.Chat<? extends CommandSender, ?>> CHAT = Facet.of(new Supplier[]{() -> {
      return new ViaFacet.Chat(Player.class, VIA);
   }, () -> {
      return new CraftBukkitFacet.Chat1_19_3();
   }, () -> {
      return new CraftBukkitFacet.Chat();
   }, () -> {
      return new BukkitFacet.Chat();
   }});
   private static final Collection<Facet.ActionBar<Player, ?>> ACTION_BAR = Facet.of(new Supplier[]{() -> {
      return new ViaFacet.ActionBarTitle(Player.class, VIA);
   }, () -> {
      return new ViaFacet.ActionBar(Player.class, VIA);
   }, () -> {
      return new CraftBukkitFacet.ActionBar_1_17();
   }, () -> {
      return new CraftBukkitFacet.ActionBar();
   }, () -> {
      return new CraftBukkitFacet.ActionBarLegacy();
   }});
   private static final Collection<Facet.Title<Player, ?, ?, ?>> TITLE = Facet.of(new Supplier[]{() -> {
      return new ViaFacet.Title(Player.class, VIA);
   }, () -> {
      return new CraftBukkitFacet.Title_1_17();
   }, () -> {
      return new CraftBukkitFacet.Title();
   }});
   private static final Collection<Facet.Sound<Player, Vector>> SOUND = Facet.of(new Supplier[]{() -> {
      return new BukkitFacet.SoundWithCategory();
   }, () -> {
      return new BukkitFacet.Sound();
   }});
   private static final Collection<Facet.EntitySound<Player, Object>> ENTITY_SOUND = Facet.of(new Supplier[]{() -> {
      return new CraftBukkitFacet.EntitySound_1_19_3();
   }, () -> {
      return new CraftBukkitFacet.EntitySound();
   }});
   private static final Collection<Facet.Book<Player, ?, ?>> BOOK = Facet.of(new Supplier[]{() -> {
      return new CraftBukkitFacet.BookPost1_13();
   }, () -> {
      return new CraftBukkitFacet.Book1_13();
   }, () -> {
      return new CraftBukkitFacet.BookPre1_13();
   }});
   private static final Collection<Facet.BossBar.Builder<Player, ?>> BOSS_BAR = Facet.of(new Supplier[]{() -> {
      return new ViaFacet.BossBar.Builder(Player.class, VIA);
   }, () -> {
      return new ViaFacet.BossBar.Builder1_9_To_1_15(Player.class, VIA);
   }, () -> {
      return new CraftBukkitFacet.BossBar.Builder();
   }, () -> {
      return new BukkitFacet.BossBarBuilder();
   }, () -> {
      return new CraftBukkitFacet.BossBarWither.Builder();
   }});
   private static final Collection<Facet.TabList<Player, ?>> TAB_LIST = Facet.of(new Supplier[]{() -> {
      return new ViaFacet.TabList(Player.class, VIA);
   }, () -> {
      return new PaperFacet.TabList();
   }, () -> {
      return new CraftBukkitFacet.TabList();
   }, () -> {
      return new BukkitFacet.TabList();
   }});
   private static final Collection<Facet.Pointers<? extends CommandSender>> POINTERS = Facet.of(new Supplier[]{() -> {
      return new BukkitFacet.CommandSenderPointers();
   }, () -> {
      return new BukkitFacet.ConsoleCommandSenderPointers();
   }, () -> {
      return new BukkitFacet.PlayerPointers();
   }});
   private final @NotNull Plugin plugin;
   private @Nullable Locale locale;

   BukkitAudience(final @NotNull Plugin plugin, final FacetAudienceProvider<?, ?> provider, final @NotNull Collection<CommandSender> viewers) {
      super(provider, viewers, CHAT, ACTION_BAR, TITLE, SOUND, ENTITY_SOUND, BOOK, BOSS_BAR, TAB_LIST, POINTERS);
      this.plugin = plugin;
   }

   void locale(final @Nullable Locale locale) {
      boolean changed = this.locale != (this.locale = locale);
      if (changed) {
         this.refresh();
      }

   }

   @Nullable Locale locale() {
      return this.locale;
   }

   protected void contributePointers(final Pointers.Builder builder) {
      builder.withDynamic(Identity.LOCALE, this::locale);
   }

   public void showBossBar(final @NotNull BossBar bar) {
      PLUGIN.set(this.plugin);
      super.showBossBar(bar);
      PLUGIN.set((Object)null);
   }
}
